/**********************************************************************
**  Copyright 2003 Steve Essery (tecknobabble@hotmail.com)
**
**  This file is part of the TaskList Applet.
**
**  The TaskList Applet is free software; you can redistribute it
**  and/or modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2 of
**  the License, or (at your option) any later version.
**
**  The TaskList Applet is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with the TaskList Applet; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**********************************************************************/

#include "custommenuwidget.h"
#include "simplebutton.h"
#include <qpe/resource.h>
#include <qfontmetrics.h>
#include <qpainter.h>
#include <qsizepolicy.h>
#include <qpe/qpeapplication.h>

void CustomMenuWidget::createLabel()
{
    int w,h=0;
    int bm = QPEApplication::style().buttonMargin()/2;

    if (style == TextAndIcon)
    {
        w = itemPixmap.width() + bm;
	h = itemPixmap.height() + bm;
	
	QFontMetrics fm = fontMetrics();
	QRect rect = fm.boundingRect(itemText);

	w += rect.width();
	if (h < rect.height())
	    h = rect.height();

	w += 4;
	h += 4;

	labelPixmap = new QPixmap(w, h);
	QPainter p(labelPixmap, parentWidget());
	p.begin(labelPixmap);
	p.fillRect(0, 0, w, h, QBrush(backgroundColor()));

	int x=bm;
	int y=(h-itemPixmap.height())/2;
	p.drawPixmap(x, y, itemPixmap);

	x += itemPixmap.width()+4;
	y=(h-rect.height())/2-2;

	p.drawText(x, y, x+rect.width(), y+rect.height(), 
	           AlignLeft|AlignVCenter, itemText);
	p.end();
    }
}


CustomMenuWidget::CustomMenuWidget(QString &text, int i, bool cf, bool ex,
		                   QWidget *parent, const char *name)
    : QWidget(parent, name),
      id(i), showCancel(cf), expandItem(ex),
      layout(NULL), itemButton(NULL), cancelButton(NULL),
      labelPixmap(NULL), itemText(text), itemPixmap(NULL), cancelPixmap(NULL)
{
    // TextOnly constructor
    style = TextOnly;

    layout = new QHBoxLayout(this, 0, 1);

    itemButton = new SimpleButton(this);
    itemButton->setAlignment(AlignLeft|AlignVCenter);
    if (expandItem)
	itemButton->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,
					      QSizePolicy::Preferred));
    itemButton->setText(itemText);
    connect(itemButton, SIGNAL(clicked()), this, SLOT(raiseClicked()));
    layout->addWidget(itemButton);

    if (showCancel)
    {
        cancelPixmap = Resource::loadPixmap("tasklist/cancel");
	cancelButton = new SimpleButton(this);
	cancelButton->setPixmap(cancelPixmap);
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelClicked()));
	layout->addWidget(cancelButton);
    }
}

CustomMenuWidget::CustomMenuWidget(QPixmap &icon, int i, bool cf, bool ex,
                                   QWidget *parent, const char *name)
    : QWidget(parent, name),
      id(i), showCancel(cf), expandItem(ex),
      layout(NULL), itemButton(NULL), cancelButton(NULL),
      labelPixmap(NULL), itemText(NULL), itemPixmap(icon), cancelPixmap(NULL)
{
    // IconOnly constructor
    style = IconOnly;

    layout = new QHBoxLayout(this, 0, 1);

    itemButton = new SimpleButton(this);
    itemButton->setAlignment(AlignLeft|AlignVCenter);
    if (expandItem)
	itemButton->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,
					      QSizePolicy::Preferred));
    itemButton->setPixmap(icon);
    connect(itemButton, SIGNAL(clicked()), this, SLOT(raiseClicked()));
    layout->addWidget(itemButton);

    if (showCancel)
    {
        cancelPixmap = Resource::loadPixmap("tasklist/cancel");
	cancelButton = new SimpleButton(this);
	cancelButton->setPixmap(cancelPixmap);
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelClicked()));
	layout->addWidget(cancelButton);
    }
}

CustomMenuWidget::CustomMenuWidget(QPixmap &icon, QString &text, int i,
                                   bool cf, bool ex,
		                   QWidget *parent, const char *name)
    : QWidget(parent, name),
      id(i), showCancel(cf), expandItem(ex),
      layout(NULL), itemButton(NULL), cancelButton(NULL),
      labelPixmap(NULL), itemText(text), itemPixmap(icon), cancelPixmap(NULL)
{
    // TextAndIcon constructor
    style = TextAndIcon;
    if (!labelPixmap)
        createLabel();

    layout = new QHBoxLayout(this, 0, 1);

    itemButton = new SimpleButton(this);
    itemButton->setAlignment(AlignLeft|AlignVCenter);
    if (expandItem)
      itemButton->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,
					    QSizePolicy::Preferred));
    itemButton->setPixmap(*labelPixmap);
    connect(itemButton, SIGNAL(clicked()), this, SLOT(raiseClicked()));
    layout->addWidget(itemButton);

    if (showCancel)
    {
        cancelPixmap = Resource::loadPixmap("tasklist/cancel");
	cancelButton = new SimpleButton(this);
	cancelButton->setPixmap(cancelPixmap);
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelClicked()));
	layout->addWidget(cancelButton);
    }
}

CustomMenuWidget::~CustomMenuWidget()
{
    if (labelPixmap)  delete labelPixmap;
    if (cancelButton) delete cancelButton;
    if (itemButton)   delete itemButton;
    if (layout)       delete layout;
}

void CustomMenuWidget::setId(int i)
{
    id = i;
}

CustomMenuWidget::CustomStyle CustomMenuWidget::customStyle()
{
    return style;
}

void CustomMenuWidget::raiseClicked()
{
    emit raisePicked(id);
}

void CustomMenuWidget::cancelClicked()
{
    emit cancelPicked(id);
}
