/**********************************************************************
**  Copyright 2003 Steve Essery (tecknobabble@hotmail.com)
**
**  This file is part of the TaskList Applet.
**
**  The TaskList Applet is free software; you can redistribute it
**  and/or modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2 of
**  the License, or (at your option) any later version.
**
**  The TaskList Applet is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with the TaskList Applet; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**********************************************************************/

#ifndef TASKLIST_APPLET_H
#define TASKLIST_APPLET_H

#include <qcopchannel_qws.h>
#include <qwidget.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qlist.h>
#include <qdict.h>
#include <qintdict.h>
#include <qlayout.h>
#include <qtimer.h>
#include <qguardedptr.h>

typedef QIntDict<QString> IDMap;
typedef QDict<QString> ChannelMap;

class TaskListApplet : public QWidget
{
    Q_OBJECT
public:
    TaskListApplet( QWidget *parent = 0, const char *name=0 );
    ~TaskListApplet();

protected:
    void mousePressEvent( QMouseEvent *);
    void paintEvent( QPaintEvent* );
    void readConfig();
    void rebuildMenu();
    void nextApp();
    void prevApp();
    void terminateApp(QString);
    void rebuildTaskBar();
    void insertIconItem(QPixmap mPixmap, int mId);
    void insertTextItem(QString aName, int mId);
    void insertTextAndIconItem(QPixmap mPixmap, QString aName, int mId);
    void insertTaskBarItem(QPixmap mPixmap, int mId);

    QCopChannel *channel;

protected slots:
    void signalReceived(const QCString& msg, const QByteArray& data);
    void raiseApp(int);
    void quitApp(int);
    void hideTaskBar();

private:
    QGuardedPtr<QPopupMenu>  menu;
    IDMap                   *idMap;
    ChannelMap              *channelMap;
    QGuardedPtr<QWidget>     taskBarParent;
    QGuardedPtr<QHBoxLayout> taskBarLayout;
    QGuardedPtr<QTimer>      hideTimer;
    QGuardedPtr<QWidget>     taskBar;
    QGuardedPtr<QHBoxLayout> taskBarParentLayout;
    QPixmap                  tasklistPixmap;
    QPixmap                  cancelPixmap;
    QString                  menuContents;
    uint                     pickedId;
    int                      taskBarTimeout;
    bool                     showCancel;
    bool                     useKill;
    bool                     taskBarLeftToRight;
    bool                     debug;
    bool                     canRaiseDesktop;
};


#endif // TASKLIST_APPLET_H

